@@ -793,21 +793,23 @@ END_TEST
 START_TEST(test_db_imap_utf7_like)
 {
 	char *trythese[] = {
-		"INBOX/&BekF3AXVBd0-",
 		"Inbox",
 		"Listen/F% %/Users",
+		"xx1_&AOcA4w-o",
+		"INBOX/&BekF3AXVBd0-",
 		NULL };
 	char *getthese[] = {
-		"name LIKE BINARY '______&BekF3AXVBd0_/%' AND name LIKE 'INBOX/____________-/%'",
 		"name LIKE 'Inbox/%'",
 		"name LIKE 'Listen/F% %/Users/%'",
+		"name LIKE BINARY '___\\\\_&AOcA4w__/%' AND name LIKE 'xx1\\\\________-o/%'",
+		"name LIKE BINARY '______&BekF3AXVBd0_/%' AND name LIKE 'INBOX/____________-/%'",
 		NULL };
 	int i;
 
 	for (i = 0; trythese[i] != NULL; i++) {
 		char *result = db_imap_utf7_like("name", trythese[i], "/%");
 
-		fail_unless(strcmp(result, getthese[i])==0, "Failed to make db_imap_utf7_like string for [%s]", trythese[i]);
+		fail_unless(strcmp(result, getthese[i])==0, "Failed to make db_imap_utf7_like string for [%s]:\n[%s] !=\n[%s]", trythese[i], result, getthese[i]);
 
 		g_free(result);
 	}
@@ -845,6 +847,18 @@ END_TEST
 //int db_findmailbox_by_regex(u64_t owner_idnr, const char *pattern,
 //			    u64_t ** children, unsigned *nchildren,
 //			    int only_subscribed);
+START_TEST(test_db_findmailbox_by_regex)
+{
+	int result;
+	u64_t *children = NULL;
+	u64_t mailbox_id = 0;
+	unsigned nchildren = 0;
+
+	result = db_createmailbox("INBOX/Trash", testidnr, &mailbox_id);
+	result = db_findmailbox_by_regex(testidnr, "INBOX/Trash", &children, &nchildren, 0);
+
+}
+END_TEST
 /**
  * \brief get info on a mailbox. Info is filled in in the
  *        mailbox_t struct.
@@ -916,7 +930,7 @@ END_TEST
  */
 START_TEST(test_db_mailbox_create_with_parents)
 {
-	u64_t mailbox_idnr = 0;
+	u64_t mailbox_idnr = 0, mb1 = 0, mb2 = 0, mb3 = 0;
 	const char *message;
 	int result;
 	int only_empty = 0, update_curmail_size = 0 ;
@@ -937,6 +951,22 @@ START_TEST(test_db_mailbox_create_with_parents)
 	result = db_delete_mailbox(mailbox_idnr, only_empty, update_curmail_size);
 	fail_unless(result == 0, "Failed at db_findmailbox or db_delete_mailbox");
 
+	result = db_mailbox_create_with_parents("INBOX/Foo/Bar-Baz", BOX_COMMANDLINE, useridnr, &mb1, &message);
+	fail_unless(result == 0 && mb1 != 0, "Failed at db_mailbox_create_with_parents: [%s]", message);
+
+	result = db_mailbox_create_with_parents("INBOX/Foo/Bar_Baz", BOX_COMMANDLINE, useridnr, &mb2, &message);
+	fail_unless(result == 0 && mb2 != 0, "Failed at db_mailbox_create_with_parents: [%s]", message);
+
+	result = db_mailbox_create_with_parents("INBOX/Foo/Bar=Baz", BOX_COMMANDLINE, useridnr, &mb3, &message);
+	fail_unless(result == 0 && mb3 != 0, "Failed at db_mailbox_create_with_parents: [%s]", message);
+
+	result = db_delete_mailbox(mb1, only_empty, update_curmail_size);
+	fail_unless(result == 0, "Failed at db_findmailbox or db_delete_mailbox");
+	result = db_delete_mailbox(mb2, only_empty, update_curmail_size);
+	fail_unless(result == 0, "Failed at db_findmailbox or db_delete_mailbox");
+	result = db_delete_mailbox(mb3, only_empty, update_curmail_size);
+	fail_unless(result == 0, "Failed at db_findmailbox or db_delete_mailbox");
+
 	result = db_findmailbox("INBOX/FOO", useridnr, &mailbox_idnr);
 	fail_unless(result == 1 && mailbox_idnr != 0, "Failed at db_findmailbox(\"INBOX/FOO\")");
 	result = db_delete_mailbox(mailbox_idnr, only_empty, update_curmail_size);
@@ -947,6 +977,7 @@ START_TEST(test_db_mailbox_create_with_parents)
 	result = db_delete_mailbox(mailbox_idnr, only_empty, update_curmail_size);
 	fail_unless(result == 0, "We just deleted inbox. Something silly.");
 	/* Cool, we've cleaned up after ourselves. */
+
 }
 END_TEST
 
@@ -1184,6 +1215,7 @@ Suite *dbmail_db_suite(void)
 	tcase_add_test(tc_db, test_db_mailbox_set_permission);
 	tcase_add_test(tc_db, test_db_imap_utf7_like);
 	tcase_add_test(tc_db, test_db_replycache);
+	tcase_add_test(tc_db, test_db_findmailbox_by_regex);
 	return s;
 }
 
