@@ -554,7 +554,9 @@ END_TEST
 
 START_TEST(test_imap_get_envelope_koi)
 {
-	char *t,*q, *w;
+	char *t;
+	const char *exp = "(\"Thu, 01 Jan 1970 00:00:00 +0000\" \"test\" ((\"=?koi8-r?Q?=E1=CE=D4=CF=CE=20=EE=C5=C8=CF=D2=CF=DB=C9=C8=20?=\" NIL \"bad\" \"foo.ru\")) ((\"=?koi8-r?Q?=E1=CE=D4=CF=CE=20=EE=C5=C8=CF=D2=CF=DB=C9=C8=20?=\" NIL \"bad\" \"foo.ru\")) ((\"=?koi8-r?Q?=E1=CE=D4=CF=CE=20=EE=C5=C8=CF=D2=CF=DB=C9=C8=20?=\" NIL \"bad\" \"foo.ru\")) ((NIL NIL \"nobody\" \"foo.ru\")) NIL NIL NIL NIL)";
+
 	struct DbmailMessage *m = dbmail_message_new();
 	GString *s = g_string_new(encoded_message_koi);
 
@@ -562,23 +564,17 @@ START_TEST(test_imap_get_envelope_koi)
 	g_string_free(s,TRUE);
 	
 	t = imap_get_envelope(GMIME_MESSAGE(m->content));
-	/* full-circle? */
-	q = g_mime_utils_header_decode_text((unsigned char *)t);
-	w = g_mime_utils_header_encode_text((unsigned char *)q);
 	
-	fail_unless(strcmp(t,w)==0,"encode/decode/encode loop failed");
+	fail_unless(strcmp(t,exp)==0,"encode/decode/encode loop failed\n[%s] !=\n[%s]", t,exp);
 
 	g_free(t);
-	g_free(q);
-	g_free(w);
 	dbmail_message_free(m);
-	
 }
 END_TEST
 
 
 			
-#define F(a,b) fail_unless(strcmp(c = imap_cleanup_address(a), b)==0, "\n[" a "] should have yielded \n[" b "] but got \n[%s]", c)
+#define F(a,b) fail_unless(strcmp(c = imap_cleanup_address(a), b)==0, "\n[%s] should have yielded \n[%s] but got \n[%s]", a,b,c)
 #define Fnull(a,b) fail_unless(strcmp(c = imap_cleanup_address(a), b)==0, "\n[] should have yielded \n[" b "] but got \n[%s]", c)
 	
 START_TEST(test_imap_cleanup_address)
@@ -599,6 +595,12 @@ START_TEST(test_imap_cleanup_address)
 	F("=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\n"
 	"	<fordfrog@gentoo.org>\n", "\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" <fordfrog@gentoo.org>");
 
+	F("=?iso-8859-1?Q?::_=5B_Arrty_=5D_::_=5B_Roy_=28L=29_St=E8phanie_=5D?=  <over.there@hotmail.com>",
+		"\"=?iso-8859-1?Q?::_=5B_Arrty_=5D_::_=5B_Roy_=28L=29_St=E8phanie_=5D?=\" <over.there@hotmail.com>");
+
+	F("\"First Address\" <first@foo.com>, =?iso-8859-1?Q?::_=5B_Arrty_=5D_::_=5B_Roy_=28L=29_St=E8phanie_=5D?=  <over.there@hotmail.com>",
+		"\"First Address\" <first@foo.com>, \"=?iso-8859-1?Q?::_=5B_Arrty_=5D_::_=5B_Roy_=28L=29_St=E8phanie_=5D?=\" <over.there@hotmail.com>");
+
 
 }
 END_TEST
@@ -631,8 +633,8 @@ START_TEST(test_imap_get_envelope_latin)
 	s = g_string_new(encoded_message_latin_2);
 	m = dbmail_message_init_with_string(m, s);
 	g_string_free(s,TRUE);
-	
-	strncpy(expect,"(\"Thu, 01 Jan 1970 00:00:00 +0000\" \"=?ISO-8859-2?Q?Re=3A_=5Bgentoo-dev=5D_New_developer=3A__?= =?ISO-8859-2?Q?Miroslav_=A9ulc_=28fordfrog=29?=\" ((\"Miroslav  =?iso-8859-2?b?qXVsYw==?=  (fordfrog)\" NIL \"fordfrog\" \"gentoo.org\")) ((\"Miroslav  =?iso-8859-2?b?qXVsYw==?=  (fordfrog)\" NIL \"fordfrog\" \"gentoo.org\")) ((\"Miroslav  =?iso-8859-2?b?qXVsYw==?=  (fordfrog)\" NIL \"fordfrog\" \"gentoo.org\")) ((NIL NIL \"gentoo-dev\" \"lists.gentoo.org\")) NIL NIL NIL NIL)",1024);
+
+	strncpy(expect,"(\"Thu, 01 Jan 1970 00:00:00 +0000\" \"=?ISO-8859-2?Q?Re=3A_=5Bgentoo-dev=5D_New_developer=3A__?= =?ISO-8859-2?Q?Miroslav_=A9ulc_=28fordfrog=29?=\" ((\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" NIL \"fordfrog\" \"gentoo.org\")) ((\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" NIL \"fordfrog\" \"gentoo.org\")) ((\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" NIL \"fordfrog\" \"gentoo.org\")) ((NIL NIL \"gentoo-dev\" \"lists.gentoo.org\")) NIL NIL NIL NIL)",1024);
 
 	t = imap_get_envelope(GMIME_MESSAGE(m->content));
 	fail_unless(strcmp(t,expect)==0,"imap_get_envelope failed\n%s\n%s\n ", expect, t);
@@ -641,6 +643,22 @@ START_TEST(test_imap_get_envelope_latin)
 	g_free(expect);
 	dbmail_message_free(m);
 
+	/*  */
+	m = dbmail_message_new();
+	s = g_string_new(encoded_message_utf8);
+	m = dbmail_message_init_with_string(m, s);
+	g_string_free(s,TRUE);
+
+	//strncpy(expect,"(\"Thu, 01 Jan 1970 00:00:00 +0000\" \"=?ISO-8859-2?Q?Re=3A_=5Bgentoo-dev=5D_New_developer=3A__?= =?ISO-8859-2?Q?Miroslav_=A9ulc_=28fordfrog=29?=\" ((\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" NIL \"fordfrog\" \"gentoo.org\")) ((\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" NIL \"fordfrog\" \"gentoo.org\")) ((\"=?ISO-8859-2?Q?=22Miroslav_=A9ulc_=28fordfrog=29=22?=\" NIL \"fordfrog\" \"gentoo.org\")) ((NIL NIL \"gentoo-dev\" \"lists.gentoo.org\")) NIL NIL NIL NIL)",1024);
+
+	t = imap_get_envelope(GMIME_MESSAGE(m->content));
+	//fail_unless(strcmp(t,expect)==0,"imap_get_envelope failed\n%s\n%s\n ", expect, t);
+	
+	g_free(t);
+	//g_free(expect);
+	dbmail_message_free(m);
+
+
 
 }
 END_TEST
@@ -658,7 +676,18 @@ START_TEST(test_imap_get_partspec)
 
 	object = imap_get_partspec(GMIME_OBJECT(message->content),"HEADER");
 	result = imap_get_logical_part(object,"HEADER");
-	fail_unless(strlen(result)==169,"imap_get_partspec failed");
+	//"From nobody Wed Sep 14 16:47:48 2005\n" 
+	expect = g_strdup("Content-Type: text/plain; charset=\"us-ascii\"\n"
+	"MIME-Version: 1.0\n"
+	"Content-Transfer-Encoding: 7bit\n"
+	"To: testuser@foo.org\n"
+	"From: somewher@foo.org\n"
+	"Subject: dbmail test message\n"
+	"\n");
+
+	fail_unless(MATCH(expect,result),"imap_get_partsec failed [%s] !=\n[%s]\n", expect, result);
+	fail_unless(strlen(result)==169,"imap_get_partspec failed [%d] [%s]", strlen(result), result);
+	g_free(expect);
 	g_free(result);
 
 	object = imap_get_partspec(GMIME_OBJECT(message->content),"TEXT");
