@@ -83,6 +83,10 @@ void dbmail_mailbox_free(struct DbmailMailbox *self)
 		g_free(self->fi);
 		self->fi = NULL;
 	}
+	if (self->charset) {
+		g_free(self->charset);
+		self->charset = NULL;
+	}
 	
 	g_free(self);
 }
@@ -226,7 +230,8 @@ static size_t dump_message_to_stream(struct DbmailMessage *message, GMimeStream
 	GString *date;
 	InternetAddressList *ialist;
 	InternetAddress *ia;
-	
+	GMimeStream *fstream;
+	GMimeFilter *filter;
 	GString *t;
 	
 	g_return_val_if_fail(GMIME_IS_MESSAGE(message->content),0);
@@ -262,13 +267,20 @@ static size_t dump_message_to_stream(struct DbmailMessage *message, GMimeStream
 		
 	}
 	
-	r += g_mime_stream_write_string(ostream,s);
-	r += g_mime_stream_write_string(ostream,"\n");
-	
+	fstream = g_mime_stream_filter_new_with_stream(ostream);
+	filter = g_mime_filter_from_new(GMIME_FILTER_FROM_MODE_DEFAULT);
+	g_mime_stream_filter_add((GMimeStreamFilter *) fstream, filter);
+	g_object_unref(filter);
+
+	r += g_mime_stream_write_string(fstream,s);
+	r += g_mime_stream_write_string(fstream,"\n");
+
+	g_object_unref(fstream);
 	g_free(s);
 	return r;
 }
 
+/* Caller must fclose the file pointer itself. */
 int dbmail_mailbox_dump(struct DbmailMailbox *self, FILE *file)
 {
 	unsigned i,j;
@@ -278,6 +290,8 @@ int dbmail_mailbox_dump(struct DbmailMailbox *self, FILE *file)
 	GList *ids, *cids = NULL, *slice, *topslice;
 	struct DbmailMessage *message = NULL;
 	GString *q, *t;
+	char *internal_date = NULL;
+	char * date2char;
 
 	if (self->ids==NULL || g_tree_nnodes(self->ids) == 0) {
 		TRACE(TRACE_DEBUG,"cannot dump empty mailbox");
@@ -289,7 +303,8 @@ int dbmail_mailbox_dump(struct DbmailMailbox *self, FILE *file)
 	q = g_string_new("");
 	t = g_string_new("");
 	ostream = g_mime_stream_file_new(file);
-	
+	g_mime_stream_file_set_owner ((GMimeStreamFile *)ostream, FALSE);
+
 	ids = g_tree_keys(self->ids);
 
 	while (ids) {
@@ -305,13 +320,16 @@ int dbmail_mailbox_dump(struct DbmailMailbox *self, FILE *file)
 	g_list_destroy(cids);
 	
 	g_list_free(g_list_first(ids));
+	date2char = date2char_str("internal_date");
 
 	while (slice) {
-		g_string_printf(q,"SELECT is_header,messageblk FROM %smessageblks b "
-				"JOIN %smessages m USING (physmessage_id) "
+		g_string_printf(q,"SELECT is_header,messageblk,%s FROM %smessageblks b "
+				"JOIN %sphysmessage p ON b.physmessage_id = p.id "
+				"JOIN %smessages m ON m.physmessage_id = p.id "
 				"WHERE message_idnr IN (%s) "
 				"ORDER BY messageblk_idnr ",
-				DBPFX, DBPFX,
+				date2char,
+				DBPFX, DBPFX, DBPFX,
 				(char *)slice->data);
 		
 		if (db_query(q->str) == -1) {
@@ -331,10 +349,14 @@ int dbmail_mailbox_dump(struct DbmailMailbox *self, FILE *file)
 				if (t->len > 0) {
 					message = dbmail_message_new();
 					message = dbmail_message_init_with_string(message,t);
+					if (internal_date && strlen(internal_date))
+						dbmail_message_set_internal_date(message, internal_date);
 					if(dump_message_to_stream(message,ostream) > 0)
 						count++;
 					dbmail_message_free(message);
 				}
+				if (internal_date) g_free(internal_date);
+				internal_date = g_strdup(db_get_result(i,2));
 				g_string_printf(t,"%s", db_get_result(i,1));
 			} else {
 				g_string_append(t, db_get_result(i,1));
@@ -347,11 +369,14 @@ int dbmail_mailbox_dump(struct DbmailMailbox *self, FILE *file)
 		
 		slice = g_list_next(slice);
 	}
+	g_free(date2char);
 	
 	if (t->len) {
 		message = dbmail_message_new();
 		message = dbmail_message_init_with_string(message,t);
-		if (dump_message_to_stream(message,ostream) > 0)
+		dbmail_message_set_internal_date(message, internal_date);
+		if (internal_date) g_free(internal_date);
+		if (dump_message_to_stream(message, ostream) > 0)
 			count++;
 		dbmail_message_free(message);
 	}
@@ -709,7 +734,7 @@ static int _handle_search_args(struct DbmailMailbox *self, char **search_keys, u
 	if (! (search_keys && search_keys[*idx]))
 		return 1;
 
-	char *t = NULL, *key = search_keys[*idx];
+	char *p = NULL, *t = NULL, *key = search_keys[*idx];
 
 	search_key_t *value = g_new0(search_key_t,1);
 	
@@ -820,7 +845,13 @@ static int _handle_search_args(struct DbmailMailbox *self, char **search_keys, u
 	 * HEADER search keys
 	 */
 #define IMAP_SET_SEARCH		(*idx)++; \
-		t = dm_stresc(search_keys[*idx]); \
+		if ((p = dbmail_iconv_str_to_db((const char *)search_keys[*idx], self->charset)) != NULL) {  \
+			t = dm_stresc(p); \
+			g_free(p); \
+		} else { \
+			TRACE(TRACE_WARNING, "search_key [%s] is not charset [%s]", search_keys[*idx], self->charset); \
+			t = dm_stresc(search_keys[*idx]); \
+		} \
 		strncpy(value->search, t, MAX_SEARCH_LEN); \
 		g_free(t); \
 		(*idx)++
@@ -1048,8 +1079,10 @@ static int _handle_search_args(struct DbmailMailbox *self, char **search_keys, u
 		
 	/* ignore the charset. Let the database handle this */
         } else if ( MATCH(key, "charset") )  {
-                (*idx)++;
-                (*idx)++; // FIXME: should we check for valid charset here?
+                (*idx)++;// FIXME: check for valid charset here
+		self->charset = g_strdup(search_keys[*idx]);
+		TRACE(TRACE_DEBUG,"using charset [%s] for searching", self->charset);
+                (*idx)++; 
 	} else {
 		/* unknown search key */
 		TRACE(TRACE_DEBUG,"unknown search key [%s]", key);
@@ -1201,6 +1234,7 @@ static GTree * mailbox_search(struct DbmailMailbox *self, search_key_t *s)
 	u64_t id;
 	char gt_lt = 0;
 	const char *op;
+	char partial[DEF_FRAGSIZE];
 	
 	GString *t;
 	GString *q;
@@ -1244,33 +1278,38 @@ static GTree * mailbox_search(struct DbmailMailbox *self, search_key_t *s)
 			
 		case IST_HDR:
 		
+		memset(partial,0,sizeof(partial));
+		snprintf(partial, DEF_FRAGSIZE, db_get_sql(SQL_PARTIAL), "headervalue");
 		g_string_printf(q, "SELECT message_idnr FROM %smessages m "
 			 "JOIN %sphysmessage p ON m.physmessage_id=p.id "
 			 "JOIN %sheadervalue v ON v.physmessage_id=p.id "
 			 "JOIN %sheadername n ON v.headername_id=n.id "
 			 "WHERE mailbox_idnr = %llu "
 			 "AND status IN (%d,%d) "
-			 "AND headername %s '%s' AND headervalue %s '%%%s%%' "
+			 "AND headername %s '%s' AND %s %s '%%%s%%' "
 			 "ORDER BY message_idnr", 
 			 DBPFX, DBPFX, DBPFX, DBPFX,
 			 dbmail_mailbox_get_id(self), 
 			 MESSAGE_STATUS_NEW, MESSAGE_STATUS_SEEN, 
 			 db_get_sql(SQL_INSENSITIVE_LIKE), s->hdrfld, 
-			 db_get_sql(SQL_INSENSITIVE_LIKE), s->search);
+			 partial, db_get_sql(SQL_INSENSITIVE_LIKE), s->search);
 			break;
 
 		case IST_DATA_TEXT:
+
+		memset(partial,0,sizeof(partial));
+		snprintf(partial, DEF_FRAGSIZE, db_get_sql(SQL_PARTIAL), "headervalue");
 		g_string_printf(q, "SELECT message_idnr FROM %smessages m "
 			"JOIN %sphysmessage p ON m.physmessage_id=p.id "
 			"JOIN %sheadervalue v on v.physmessage_id=p.id "
 			"WHERE mailbox_idnr = %llu "
 			"AND status IN (%d,%d) "
-			"AND headervalue %s '%%%s%%' "
+			"AND %s %s '%%%s%%' "
 			"ORDER BY message_idnr",
 			DBPFX, DBPFX, DBPFX, 
 			dbmail_mailbox_get_id(self),
 			MESSAGE_STATUS_NEW, MESSAGE_STATUS_SEEN,
-			db_get_sql(SQL_INSENSITIVE_LIKE), s->search);
+			partial, db_get_sql(SQL_INSENSITIVE_LIKE), s->search);
 			break;
 			
 		case IST_IDATE:
@@ -1298,7 +1337,7 @@ static GTree * mailbox_search(struct DbmailMailbox *self, search_key_t *s)
 			DBPFX, DBPFX, DBPFX,
 			dbmail_mailbox_get_id(self),
 			MESSAGE_STATUS_NEW, MESSAGE_STATUS_SEEN,
-			t->str, db_get_sql(SQL_INSENSITIVE_LIKE), s->search);
+			t->str, db_get_sql(SQL_SENSITIVE_LIKE), s->search);
 		break;
 
 		case IST_SIZE_LARGER:
@@ -1344,8 +1383,6 @@ static GTree * mailbox_search(struct DbmailMailbox *self, search_key_t *s)
 		
 		
 		for (i=0; i < rows; i++) {
-			k = g_new0(u64_t,1);
-			v = g_new0(u64_t,1);
 			id = db_get_result_u64(i,0);
 			if (! (w = g_tree_lookup(self->ids, &id))) {
 				TRACE(TRACE_ERROR, "key missing in self->ids: [%llu]\n", id);
@@ -1353,6 +1390,8 @@ static GTree * mailbox_search(struct DbmailMailbox *self, search_key_t *s)
 			}
 			assert(w);
 			
+			k = g_new0(u64_t,1);
+			v = g_new0(u64_t,1);
 			*k = id;
 			*v = *w;
 
@@ -1384,10 +1423,14 @@ GTree * dbmail_mailbox_get_set(struct DbmailMailbox *self, const char *set, gboo
 
 	b = g_tree_new_full((GCompareDataFunc)ucmp,NULL, (GDestroyNotify)g_free, (GDestroyNotify)g_free);
 
+	if (g_tree_nnodes(self->ids) == 0)
+		return b;
+
 	TRACE(TRACE_DEBUG,"[%s] uid [%d]", set, uid);
 	
 	if (uid) {
 		ids = g_tree_keys(self->ids);
+		assert(ids);
 		ids = g_list_last(ids);
 		hi = *((u64_t *)ids->data);
 		ids = g_list_first(ids);
@@ -1421,10 +1464,11 @@ GTree * dbmail_mailbox_get_set(struct DbmailMailbox *self, const char *set, gboo
 			if (strlen(rest) > 1)
 				rest++;
 		} else {
-			if (! (l = strtoull(sets->data,&rest,10))) {
+			if (! (l = dm_strtoull(sets->data,&rest,10))) {
 				error = TRUE;
 				break;
 			}
+
 			if (l == 0xffffffff) // outlook
 				l = hi;
 
@@ -1435,10 +1479,16 @@ GTree * dbmail_mailbox_get_set(struct DbmailMailbox *self, const char *set, gboo
 		if (rest[0]==':') {
 			if (strlen(rest)>1)
 				rest++;
+
 			if (rest[0] == '*') 
 				r = hi;
 			else {
-				r = strtoull(rest,NULL,10);
+				
+				if (! (r = dm_strtoull(rest,NULL,10))) {
+					error = TRUE;
+					break;
+				}
+
 				if (r == 0xffffffff) // outlook
 					r = hi;
 			}
