@@ -99,6 +99,9 @@ typedef struct {
 	field_t pfx;		/**< prefix for tables e.g. dbmail_ */
 	unsigned int serverid;	/**< unique id for dbmail instance used in clusters */
 	field_t encoding;	/**< character encoding to use */
+	unsigned int query_time_info;
+	unsigned int query_time_message;
+	unsigned int query_time_warning;
 } db_param_t;
 
 /** configuration items */
@@ -112,6 +115,7 @@ typedef struct {
 	char ip_src[IPNUM_LEN];	/* client IP-number */
 	field_t clientname;	/* resolved client ip */
 	int timeout;		/* server timeout (seconds) */
+	int login_timeout;	/* login timeout (seconds) */
 	void *userData;
 } clientinfo_t;
 
@@ -121,6 +125,7 @@ typedef struct {
 	int numSockets;
 	int resolveIP;
 	int timeout;
+	int login_timeout;
 	int (*ClientHandler) (clientinfo_t *);
 } ChildInfo_t;
 
@@ -231,6 +236,7 @@ enum IMAP_COMMAND_TYPES {
 	IMAP_COMM_NAMESPACE, 
 	IMAP_COMM_THREAD, 
 	IMAP_COMM_UNSELECT,
+	IMAP_COMM_IDLE,
 	IMAP_COMM_LAST
 };
 
@@ -301,6 +307,7 @@ typedef struct {
 	int maxChildren;
 	int childMaxConnect;
 	int timeout;
+	int login_timeout;
 	char **iplist; // Allocated memory.
 	int ipcount;
 	int *listenSockets; // Allocated memory.
@@ -488,13 +495,14 @@ typedef enum {
 	SQL_TO_DATETIME,
 	SQL_TO_CHAR,
 	SQL_CURRENT_TIMESTAMP,
-	SQL_REPLYCACHE_EXPIRE,
+	SQL_EXPIRE,
 	SQL_BINARY,
 	SQL_REGEXP,
 	SQL_SENSITIVE_LIKE,
 	SQL_INSENSITIVE_LIKE,
 	SQL_ENCODE_ESCAPE,
-	SQL_STRCASE
+	SQL_STRCASE,
+	SQL_PARTIAL
 		
 } sql_fragment_t;
 
