@@ -124,7 +124,35 @@ int db_disconnect(void)
 int db_check_connection(void)
 	{ return db->check_connection(); }
 int db_query(const char *the_query)
-	{ return db->query(the_query); }
+{
+	/* Timing information stuffed in here so as
+	 * to be common across all databases. */
+	int result;
+	time_t before, after;
+
+	before = time(NULL);
+	
+	result = db->query(the_query);
+
+	after = time(NULL);
+
+	if (before == (time_t)-1 || after == (time_t)-1) {
+		/* Can't log because time(2) failed. */
+	} else {
+		/* This is signed on the chance that ntpd ran during the query
+		 * so it might look like it went back in time. */
+		int elapsed = (int)((time_t) (after - before));
+		TRACE(TRACE_DEBUG, "last query took [%d] seconds", elapsed);
+		if (elapsed > (int)_db_params.query_time_info)
+			TRACE(TRACE_INFO, "slow query [%s] took [%d] seconds", the_query, elapsed);
+		if (elapsed > (int)_db_params.query_time_message)
+			TRACE(TRACE_MESSAGE, "slow query [%s] took [%d] seconds", the_query, elapsed);
+		if (elapsed > (int)_db_params.query_time_warning)
+			TRACE(TRACE_WARNING, "slow query [%s] took [%d] seconds", the_query, elapsed);
+	}
+
+	return result;
+}
 u64_t db_insert_result(const char *sequence_identifier)
 	{ return db->insert_result(sequence_identifier); }
 unsigned db_num_rows(void)
