@@ -134,7 +134,13 @@ int lmtp_handle_connection(clientinfo_t * ci)
 
 	if (ci->tx) {
 		/* sending greeting */
-		ci_write(ci->tx, "220 %s DBMail LMTP service ready to rock\r\n", myhostname);
+		field_t banner;
+		GETCONFIGVALUE("banner", "LMTP", banner);
+		if (strlen(banner) > 0) {
+			ci_write(ci->tx, "220 %s %s\r\n", myhostname, banner);
+		} else {
+			ci_write(ci->tx, "220 %s DBMail LMTP service ready to rock\r\n", myhostname);
+		}
 		fflush(ci->tx);
 	} else {
 		TRACE(TRACE_MESSAGE, "TX stream is null!");
@@ -152,7 +158,10 @@ int lmtp_handle_connection(clientinfo_t * ci)
 		}
 
 		/* set the timeout counter */
-		alarm(ci->timeout);
+		if (session.state == LHLO)
+			alarm(ci->login_timeout);
+		else
+			alarm(ci->timeout);
 
 		/* clear the buffer */
 		memset(buffer, 0, INCOMING_BUFFER_SIZE);
@@ -586,7 +595,7 @@ int lmtp(void *stream, void *instream, char *buffer,
 
 					dbmail_message_set_header(msg, "Return-Path", from.start->data);
 					if (insert_messages(msg, &rcpt) == -1) {
-						ci_write((FILE *) stream, "503 Message not received\r\n");
+						ci_write((FILE *) stream, "430 Message not received\r\n");
 					} else {
 						/* The DATA command itself it not given a reply except
 						 * that of the status of each of the remaining recipients. */
