@@ -56,7 +56,9 @@ _ldap_cfg_t _ldap_cfg;
 static GList * __auth_get_every_match(const char *q, char **retfields);
 
 static int dm_ldap_user_shadow_rename(u64_t user_idnr, const char *new_name);
-static int auth_reconnect(void);
+static int authldap_connect(void);
+static int authldap_disconnect(void);
+static int authldap_reconnect(void);
 static int auth_search(const gchar *query);
 
 static void __auth_get_config(void)
@@ -96,7 +98,6 @@ static void __auth_get_config(void)
 	/* defaults to version 3 */
 	if (!_ldap_cfg.version_int)
 		_ldap_cfg.version_int=3;
-
 	/* Compare the input string with the possible options,
 	 * making sure not to exceeed the length of the given string */
 	{
@@ -137,14 +138,19 @@ static int auth_ldap_bind(void)
 
 }
 
+/* Module api wrappers */
+int auth_connect(void)
+	{ return authldap_connect(); }
+int auth_disconnect(void)
+	{ return authldap_disconnect(); }
 /*
- * auth_connect()
+ * authldap_connect()
  *
  * initializes the connection for authentication.
  * 
  * returns 0 on success, -1 on failure
  */
-int auth_connect(void)
+static int authldap_connect(void)
 {
 	int version = 0;
 #ifdef HAVE_LDAP_INITIALIZE
@@ -152,9 +158,9 @@ int auth_connect(void)
 	char *uri;
 #endif
 
-	if (_ldap_conn != NULL) 
-		return 0;
-	
+       if (_ldap_conn != NULL)
+               return 0;
+
 	__auth_get_config();
 
 	switch (_ldap_cfg.version_int) {
@@ -215,16 +221,33 @@ int auth_connect(void)
 	return auth_ldap_bind();	
 }
 
-int auth_disconnect(void)
+static int authldap_disconnect(void) 
 {
 	/* Destroy the connection */
 	if (_ldap_conn != NULL) {
+		/* If LDAP server has gone, we will catch a SIGPIPE in 
+		 * ldap_unbind... we should ignore it.  */
+		struct sigaction act, oldact;
+
+		memset(&act, 0, sizeof(act));
+		memset(&oldact, 0, sizeof(oldact));
+		act.sa_handler = SIG_IGN;
+		sigaction(SIGPIPE, &act, &oldact);
+
 		ldap_unbind(_ldap_conn);
 		_ldap_conn = NULL;
+
+		sigaction(SIGPIPE, &oldact, 0);
 	}
 	return 0;
 }
 
+static int authldap_reconnect(void)
+{
+	authldap_disconnect();
+	return authldap_connect();
+}
+
 static int auth_search(const gchar *query)
 {
 	int c=0;
@@ -241,12 +264,12 @@ static int auth_search(const gchar *query)
 		
 		switch (_ldap_err) {
 			case LDAP_SERVER_DOWN:
-				TRACE(TRACE_ERROR, "%s", ldap_err2string(_ldap_err));
-				if (auth_reconnect())
+				TRACE(TRACE_WARNING, "LDAP gone away: %s. Try to reconnect(%d/5).", ldap_err2string(_ldap_err),c);
+				if (authldap_reconnect())
 					sleep(2); // reconnect failed. wait before trying again
 				break;
 			default:
-				TRACE(TRACE_ERROR, "%s", ldap_err2string(_ldap_err));
+				TRACE(TRACE_ERROR, "LDAP error(%d): %s", _ldap_err, ldap_err2string(_ldap_err));
 				return _ldap_err;
 				break;
 		}
@@ -256,13 +279,6 @@ static int auth_search(const gchar *query)
 	return -1;
 }
 
-
-static int auth_reconnect(void)
-{
-	auth_disconnect();
-	return auth_connect();
-}
-
 void dm_ldap_freeresult(GList *entlist)
 {
 	GList *fldlist, *attlist;
@@ -271,16 +287,19 @@ void dm_ldap_freeresult(GList *entlist)
 		fldlist = entlist->data;
 		while(fldlist) {
 			attlist = fldlist->data;
-			g_list_foreach(attlist,(GFunc)g_free,NULL);
-			g_list_free(attlist);
+			g_list_destroy(attlist);
 			if (! g_list_next(fldlist))
 				break;
 			fldlist = g_list_next(fldlist);
 		}
+		g_list_free(g_list_first(fldlist));
+
 		if (! g_list_next(entlist))
 			break;
 		entlist = g_list_next(entlist);
 	}
+
+	g_list_free(g_list_first(entlist));
 }
 
 GList * dm_ldap_entdm_list_get_values(GList *entlist)
@@ -472,10 +491,11 @@ static int dm_ldap_mod_field(u64_t user_idnr, const char *fieldname, const char
 
 	_ldap_err = ldap_modify_s(_ldap_conn, _ldap_dn, mods);
 	if (_ldap_err) {
-		TRACE(TRACE_ERROR,"error changing field [%s] to value [%s]: %s", fieldname, newvalue, ldap_err2string(_ldap_err));
+		TRACE(TRACE_ERROR,"dn: %s, %s: %s [%s]", _ldap_dn, fieldname, newvalue, ldap_err2string(_ldap_err));
 		ldap_memfree(_ldap_dn);
 		return -1;
 	}
+	TRACE(TRACE_DEBUG,"dn: %s, %s: %s", _ldap_dn, fieldname, newvalue);
 	ldap_memfree(_ldap_dn);
 	return 0;
 }
@@ -766,8 +786,9 @@ int auth_getmaxmailsize(u64_t user_idnr, u64_t * maxmail_size)
 		 user_idnr);
 	max_char = __auth_get_first_match(query, fields);
 	*maxmail_size = (max_char) ? strtoull(max_char, 0, 10) : 0;
-	if (max_char != NULL)
-		g_free(max_char);
+	
+	// if max_char is NULL g_free will not fail it simply return
+	g_free(max_char);
 
 	TRACE(TRACE_DEBUG, "returned value is [%llu]", *maxmail_size);
 
@@ -819,6 +840,31 @@ GList * auth_get_known_users(void)
 	return users;
 }
 
+/* Fills the aliases list with all existing aliases
+ * return -2 on mem error, -1 on db-error, 0 on succeses */
+GList * auth_get_known_aliases(void)
+{
+	char *query;
+	char *fields[] = { _ldap_cfg.field_uid, NULL };
+	GList *aliases;
+	GList *entlist;
+	
+	GString *t = g_string_new(_ldap_cfg.forw_objectclass);
+	GList *l = g_string_split(t,",");
+	g_string_free(t,TRUE);
+	
+	query =  dm_ldap_get_filter('&',"objectClases",l);
+	entlist = __auth_get_every_match(query, fields);
+	g_free(query);
+	
+	TRACE(TRACE_INFO, "found %d aliases", g_list_length(entlist));
+
+	aliases = dm_ldap_entdm_list_get_values(entlist);
+	
+	dm_ldap_freeresult(entlist);
+	return aliases;
+}
+
 /*
  * auth_check_user_ext()
  * 
@@ -837,8 +883,11 @@ int auth_check_user_ext(const char *address, struct dm_list *userids,
 	u64_t id;
 	char *endptr = NULL;
 	char query[AUTH_QUERY_SIZE];
-	char *fields[] = { _ldap_cfg.field_nid, _ldap_cfg.field_fwdtarget, NULL };
-	unsigned c2;
+	char *fields[] = { 
+		_ldap_cfg.field_nid, 
+		_ldap_cfg.field_fwdtarget[0] ? _ldap_cfg.field_fwdtarget : NULL, 
+		NULL 
+	};
 	char *attrvalue;
 	GList *entlist, *fldlist, *attlist;
 
@@ -883,12 +932,10 @@ int auth_check_user_ext(const char *address, struct dm_list *userids,
 	entlist = g_list_first(entlist);
 	while (entlist) {
 		fldlist = g_list_first(entlist->data);
-		c2 = 0;
 		while(fldlist) {
 			attlist = g_list_first(fldlist->data);
 			while(attlist) {
 				attrvalue = (char *)attlist->data;
-				
 				occurences += auth_check_user_ext(attrvalue, userids, fwds, checks+1);
 				
 				if (! g_list_next(attlist))
@@ -898,7 +945,6 @@ int auth_check_user_ext(const char *address, struct dm_list *userids,
 			if (! g_list_next(fldlist))
 				break;
 			fldlist = g_list_next(fldlist);
-			c2++;
 		}
 		if (! g_list_next(entlist))
 			break;
@@ -1188,6 +1234,10 @@ int auth_validate(clientinfo_t *ci, char *username, char *password, u64_t * user
 		TRACE(TRACE_DEBUG, "username or password is NULL");
 		return 0;
 	}
+	if (strlen(password) == 0) {
+		TRACE(TRACE_WARNING, "User \"%s\" try to use anonimous LDAP bind!", username);
+		return 0;
+	}
 
 	/* the shared mailbox user should not log in! */
 	if (strcmp(username, PUBLIC_FOLDER_USER) == 0)
