@@ -47,7 +47,7 @@ const char * db_get_sql(sql_fragment_t frag)
 		case SQL_CURRENT_TIMESTAMP:
 			return "CURRENT_TIMESTAMP";
 		break;
-		case SQL_REPLYCACHE_EXPIRE:
+		case SQL_EXPIRE:
 			return "NOW() - INTERVAL %d DAY";
 		break;
 		case SQL_BINARY:
@@ -64,6 +64,7 @@ const char * db_get_sql(sql_fragment_t frag)
 		break;
 		case SQL_STRCASE:
 		case SQL_ENCODE_ESCAPE:
+		case SQL_PARTIAL:
 			return "%s";
 		break;
 	}
@@ -78,7 +79,7 @@ static unsigned last_row_number = 0; /* the number of the row in last_row */
 
 
 /* database parameters */
-db_param_t _db_params;
+extern db_param_t _db_params;
 
 /* static functions, only used locally */
 
@@ -148,9 +149,6 @@ int db_connect()
 	/* connect */
 	mysql_init(&conn);
 
-	/* auto re-connect */
-	conn.reconnect = 1;
-
 	/* use the standard MySQL port by default */
 	if (_db_params.port == 0)
 		_db_params.port = DB_MYSQL_STANDARD_PORT;
@@ -168,7 +166,6 @@ int db_connect()
 			sock = _db_params.sock;
 	}
 
-
 	if (mysql_real_connect(&conn, _db_params.host, _db_params.user,
 			       _db_params.pass, _db_params.db,
 			       _db_params.port, sock, 0) == NULL) {
@@ -288,11 +285,11 @@ int db_query(const char *q)
 		return DM_EQUERY;
 	
 	TRACE(TRACE_DEBUG, "query [%s]", q);
+
 	if (mysql_real_query(&conn, q, querysize)) {
 		TRACE(TRACE_ERROR, "[%s] [%s]", mysql_error(&conn), q);
 		return DM_EQUERY;
 	}
-
 	if(res != NULL) 
 		db_free_result();
 	
