@@ -45,7 +45,7 @@ const char * db_get_sql(sql_fragment_t frag)
 		case SQL_CURRENT_TIMESTAMP:
 			return "CURRENT_TIMESTAMP";
 		break;
-		case SQL_REPLYCACHE_EXPIRE:
+		case SQL_EXPIRE:
 			return "NOW() - INTERVAL '%d DAY'";
 		break;
 		case SQL_BINARY:
@@ -66,6 +66,9 @@ const char * db_get_sql(sql_fragment_t frag)
 		case SQL_STRCASE:
 			return "LOWER(%s)";
 		break;
+		case SQL_PARTIAL:
+			return "SUBSTRING(%s,0,255)";
+		break;
 	}
 	return NULL;
 }
@@ -76,7 +79,7 @@ static void _create_binary_table(void);
 static void _free_binary_table(void);
 static void _set_binary_table(unsigned row, unsigned field);
 static char*** bintbl = NULL;
-db_param_t _db_params;
+extern db_param_t _db_params;
 
 int db_connect()
 {
