@@ -28,22 +28,19 @@
 #include <sqlite3.h>
 #define THIS_MODULE "sql"
 
-db_param_t _db_params;
+extern db_param_t _db_params;
+
 #define DBPFX _db_params.pfx
 
 static sqlite3 *conn;
 
-/* SQLITE3 internals... */
-extern int sqlite3ReadUtf8(const unsigned char *);
-extern const unsigned char sqlite3UpperToLower[];
-extern int sqlite3utf8CharLen(const char *pData, int nByte);
-
 const char * db_get_sql(sql_fragment_t frag)
 {
 	switch(frag) {
 		case SQL_ENCODE_ESCAPE:
 		case SQL_TO_CHAR:
 		case SQL_STRCASE:
+		case SQL_PARTIAL:
 			return "%s";
 		break;
 		case SQL_TO_DATE:
@@ -55,7 +52,7 @@ const char * db_get_sql(sql_fragment_t frag)
 		case SQL_CURRENT_TIMESTAMP:
 			return "STRFTIME('%Y-%m-%d %H:%M:%S','now','localtime')";
 		break;
-		case SQL_REPLYCACHE_EXPIRE:
+		case SQL_EXPIRE:
 			return "DATETIME('now','-%d DAYS')";	
 		break;
 		case SQL_BINARY:
@@ -91,25 +88,6 @@ struct qtmp {
 
 struct qtmp *lastq = 0;
 
-static void dbsqlite_current_timestamp(sqlite3_context *f, int argc UNUSED, const sqlite3_value **argv UNUSED)
-{
-	char timestr[21];
-	struct tm tm;
-	time_t now;
-
-	time(&now);
-	localtime_r(&now, &tm);
-	strftime(timestr, sizeof(timestr)-1, "%Y-%m-%d %H:%M:%S", &tm);
-	(void)sqlite3_result_text(f,timestr,-1,SQLITE_TRANSIENT);
-}
-
-static void dbsqlite_current_timestamp_unix(sqlite3_context *f, int argc UNUSED,  const sqlite3_value **argv UNUSED)
-{
-	char buf[63];
-	sprintf(buf, "%ld", time(NULL)); /* assumes time() is signed int */
-	(void)sqlite3_result_text(f,buf,-1,SQLITE_TRANSIENT);
-}
-
 /* this is lifted directly from sqlite -- cut here -- */
 struct compareInfo {
 	unsigned char matchAll;
@@ -119,7 +97,7 @@ struct compareInfo {
 };
 static const struct compareInfo likeInfo = { '%', '_',   0, 0 };
 #define sqliteNextChar(X)  while( (0xc0&*++(X))==0x80 ){}
-#define sqliteCharVal(X)   sqlite3ReadUtf8(X)
+#define sqliteCharVal(X)   g_utf8_get_char((const char *)X)
 
 static int patternCompare(
   const unsigned char *zPattern,              /* The glob pattern */
@@ -146,7 +124,7 @@ static int patternCompare(
         }
         zPattern++;
       }
-      if( c && esc && sqlite3ReadUtf8(&zPattern[1])==esc ){
+      if( c && esc && (const int)g_utf8_get_char((const char *)&zPattern[1])==esc ){
         unsigned char const *zTemp = &zPattern[1];
         sqliteNextChar(zTemp);
         c = *zTemp;
@@ -161,9 +139,9 @@ static int patternCompare(
       }else{
         while( (c2 = *zString)!=0 ){
           if( noCase ){
-            c2 = sqlite3UpperToLower[c2];
-            c = sqlite3UpperToLower[c];
-            while( c2 != 0 && c2 != c ){ c2 = sqlite3UpperToLower[*++zString]; }
+            c2 = g_ascii_tolower(c2);
+            c = g_ascii_tolower(c);
+            while( c2 != 0 && c2 != c ){ c2 = g_ascii_tolower(*++zString); }
           }else{
             while( c2 != 0 && c2 != c ){ c2 = *++zString; }
           }
@@ -207,12 +185,12 @@ static int patternCompare(
       if( c2==0 || (seen ^ invert)==0 ) return 0;
       sqliteNextChar(zString);
       zPattern++;
-    }else if( esc && !prevEscape && sqlite3ReadUtf8(zPattern)==esc){
+    }else if( esc && !prevEscape && (const int)g_utf8_get_char((const char *)zPattern)==esc){
       prevEscape = 1;
       sqliteNextChar(zPattern);
     }else{
       if( noCase ){
-        if( sqlite3UpperToLower[c] != sqlite3UpperToLower[*zString] ) return 0;
+        if( g_ascii_tolower(c) != g_ascii_tolower(*zString) ) return 0;
       }else{
         if( c != *zString ) return 0;
       }
@@ -236,12 +214,12 @@ static void dbsqlite_cslike(sqlite3_context *context, int argc, sqlite3_value **
 		** Otherwise, return an error.
 		*/
 		const unsigned char *zEsc = sqlite3_value_text(argv[2]);
-		if (sqlite3utf8CharLen((const char *)zEsc, -1) != 1) {
+		if (g_utf8_strlen((const char *)zEsc, -1) != 1) {
 			sqlite3_result_error(context, 
 				"ESCAPE expression must be a single character", -1);
 			return;
 		}
-		escape = sqlite3ReadUtf8(zEsc);
+		escape = g_utf8_get_char((const char *)zEsc);
 	}
 	if (zA && zB) {
 		sqlite3_result_int(context, patternCompare(zA, zB, &likeInfo, escape));
@@ -326,6 +304,7 @@ u64_t db_insert_result(const char *sequence_identifier UNUSED)
 int db_query(const char *the_query)
 {
 	char *errmsg;
+	int res, retry=0;
 
 	if (lastq) {
 		if (lastq->resp) sqlite3_free_table(lastq->resp);
@@ -338,12 +317,25 @@ int db_query(const char *the_query)
 	}
 	TRACE(TRACE_DEBUG,"%s", the_query);
 
-	if (sqlite3_get_table(conn, the_query, &lastq->resp,
-				(int *)&lastq->rows, (int *)&lastq->cols, &errmsg) != SQLITE_OK) {
+	while (TRUE) {
+		res = sqlite3_get_table(conn, the_query, &lastq->resp,
+			 (int *)&lastq->rows, (int *)&lastq->cols, &errmsg);
+
+		if (res == SQLITE_OK)
+			break;
+		
+		if ((res == SQLITE_BUSY || res == SQLITE_LOCKED) && (retry++ < 5)) {
+			TRACE(TRACE_DEBUG,"database locked, retrying...");
+			sqlite3_free(errmsg);
+			usleep(200 + 500*retry);
+			continue;
+		}
+
 		TRACE(TRACE_ERROR, "sqlite3_get_table failed: %s", errmsg);
 		sqlite3_free(errmsg);
 		return -1;
 	}
+
 	if (lastq->rows < 0 || lastq->cols < 0) {
 		lastq->rows = 0;
 		lastq->cols = 0;
