@@ -119,9 +119,10 @@ int serverparent_getopt(serverConfig_t *config, const char *service, int argc, c
 			}
 			break;
 		case 'f':
-			if (optarg && strlen(optarg) > 0)
+			if (optarg && strlen(optarg) > 0) {
+                                g_free(configFile);
 				configFile = g_strdup(optarg);
-			else {
+			} else {
 				fprintf(stderr, "%s: -f requires a filename argument\n\n", argv[0]);
 				return 1;
 			}
@@ -286,6 +287,18 @@ void LoadServerConfig(serverConfig_t * config, const char * const service)
 	TRACE(TRACE_DEBUG, "timeout [%d] seconds",
 	      config->timeout);
 
+	/* read items: LOGIN_TIMEOUT */
+	config_get_value("LOGIN_TIMEOUT", service, val);
+	if (strlen(val) == 0) {
+		TRACE(TRACE_DEBUG, "no value for TIMEOUT in config file");
+		config->login_timeout = 60;
+	} else if ((config->login_timeout = atoi(val)) <= 10)
+		TRACE(TRACE_FATAL, "value for TIMEOUT is invalid: [%d]",
+		      config->login_timeout);
+
+	TRACE(TRACE_DEBUG, "login_timeout [%d] seconds",
+	      config->login_timeout);
+
 	/* SOCKET */
 	config_get_value("SOCKET", service, val);
 	if (strlen(val) == 0)
@@ -391,7 +404,7 @@ void LoadServerConfig(serverConfig_t * config, const char * const service)
        config_get_value("MINSPARECHILDREN", service, val);
        if (strlen(val) == 0)
                TRACE(TRACE_FATAL, "no value for MINSPARECHILDREN in config file");
-       if ( (config->minSpareChildren = atoi(val)) <= 0)
+       if ( (config->minSpareChildren = atoi(val)) < 0)
                TRACE(TRACE_FATAL, "value for MINSPARECHILDREN is invalid: [%d]",
                        config->minSpareChildren);
 
