@@ -48,12 +48,13 @@ static int do_list(u64_t user_idnr);
 static int do_activate(u64_t user_idnr, char *name);
 static int do_deactivate(u64_t user_idnr, char *name);
 static int do_remove(u64_t user_idnr, char *name);
+static int do_edit(u64_t user_idnr, char *name);
 static int do_insert(u64_t user_idnr, char *name, char *source);
-static int do_cat(u64_t user_idnr, char *name);
+static int do_cat(u64_t user_idnr, char *name, FILE *out);
 
 int main(int argc, char *argv[])
 {
-	int res = 0, opt = 0, opt_prev = 0, act = 0;
+	int res = 0, opt = 0, opt_prev = 0;
 	u64_t user_idnr = 0;
 	char *user_name = NULL;
 	char *script_name = NULL;
@@ -61,13 +62,16 @@ int main(int argc, char *argv[])
 	extern char *optarg;
 	extern int opterr;
 
+	int activate = 0, deactivate = 0, insert = 0;
+	int remove = 0, list = 0, cat = 0, help = 0, edit = 0;
+
 	openlog(PNAME, LOG_PID, LOG_MAIL);
 	setvbuf(stdout, 0, _IONBF, 0);
 
 	/* get options */
 	opterr = 0;		/* suppress error message from getopt() */
 	while ((opt = getopt(argc, argv,
-		"-a:d:i:c::r:u:l" /* Major modes */
+		"-a::d::i:c::r:u:le::" /* Major modes */
 		/*"i"*/ "f:qnyvVh" /* Common options */ )) != -1) {
 		/* The initial "-" of optstring allows unaccompanied
 		 * options and reports them as the optarg to opt 1 (not '1') */
@@ -79,34 +83,51 @@ int main(int argc, char *argv[])
 		case -1:
 			/* Break right away if this is the end of the args */
 			break;
-		case 'a':
-		case 'd':
-		case 'i':
-		case 'r':
-			if (act != 0 && opt != opt_prev)
-				act = 'h';
-			else
-				act = opt;
+		case 'a': /* activate */
+			activate = 1;
+			goto major_script;
+		case 'd': /* deactivate */
+			deactivate = 1;
+			goto major_script;
+		case 'i': /* insert */
+			insert = 1;
+			goto major_script;
+		case 'r': /* remove */
+			remove = 1;
+			goto major_script;
 
-			if (!script_name) {
+		major_script:
+			if (!optarg) {
+				/* Need optarg */
+			} else if (!script_name) {
 				script_name = g_strdup(optarg);
 			} else if (!script_source) {
 				script_source = g_strdup(optarg);
 			}
 			break;
-		case 'c':
+		case 'e':
+			edit = 1;
+
 			if (optarg)
 				script_name = g_strdup(optarg);
-			act = opt;
+
+			break;
+		case 'c':
+			cat = 1;
+
+			if (optarg)
+				script_name = g_strdup(optarg);
+
+			/* Don't print anything but the script. */
+			quiet = 1;
+			verbose = 0;
+
 			break;
 		case 'u':
 			user_name = g_strdup(optarg);
 			break;
 		case 'l':
-			if (act != 0)
-				act = 'h';
-			else
-				act = opt;
+			list = 1;
 			break;
 
 		/* Common options */
@@ -124,7 +145,7 @@ int main(int argc, char *argv[])
 			break;
 
 		case 'h':
-			act = 'h';
+			help = 1;
 			break;
 
 		case 'n':
@@ -155,18 +176,31 @@ int main(int argc, char *argv[])
 			return 1;
 
 		default:
-			act = 'h';
+			help = 1;
 			break;
 		}
 	}
 
-	if (act == 'h' || act == 0 || !user_name || (no_to_all && yes_to_all)) {
+/*
+	if (insert) printf("got insert\n");
+	if (remove) printf("got remove\n");
+	if (list) printf("got list\n");
+	if (cat) printf("got cat\n");
+	if (activate) printf("got activate\n");
+	if (deactivate) printf("got deactivate\n");
+*/
+
+	/* Only one major mode is allowed */
+	if ((edit + insert + remove + list + cat > 1)
+	/* Only insert/edit are allowed together with activate or deactivate */
+	 || ((remove + list + cat == 1) && (activate + deactivate > 0))
+	/* You may either activate or deactivate as a mode on its own*/
+	 || (((edit + insert + remove + list + cat == 0) && (activate + deactivate != 1)))
+	 || (help || !user_name || (no_to_all && yes_to_all))) {
 		do_showhelp();
 		goto mainend;
 	}
 
-	qprintf("*** dbmail-sievecmd ***\n");
-
 	/* read the config file */
         if (config_read(configFile) == -1) {
                 qerrorf("Failed. Unable to read config file %s\n",
@@ -179,7 +213,6 @@ int main(int argc, char *argv[])
 	GetDBParams(&_db_params);
 
 	/* Open database connection */
-	qprintf("Opening connection to database...\n");
 	if (db_connect() != 0) {
 		qerrorf("Failed. Could not connect to database (check log)\n");
 		g_free(user_name);
@@ -187,15 +220,12 @@ int main(int argc, char *argv[])
 	}
 
 	/* Open authentication connection */
-	qprintf("Opening connection to authentication...\n");
 	if (auth_connect() != 0) {
 		qerrorf("Failed. Could not connect to authentication (check log)\n");
 		g_free(user_name);
 		return -1;
 	}
 
-	qprintf("Ok. Connected!\n");
-
 	/* Retrieve the user ID number */
 	switch (auth_user_exists(user_name, &user_idnr)) {
 	case 0:
@@ -209,30 +239,24 @@ int main(int argc, char *argv[])
 		goto mainend;
 	}
 
-	switch (act) {
-	case 'a':
-		res = do_activate(user_idnr, script_name);
-		break;
-	case 'd':
-		res = do_deactivate(user_idnr, script_name);
-		break;
-	case 'i':
+	if (insert)
 		res = do_insert(user_idnr, script_name, script_source);
-		break;
-	case 'c':
-		res = do_cat(user_idnr, script_name);
-		break;
-	case 'r':
+	if (remove)
 		res = do_remove(user_idnr, script_name);
-		break;
-	case 'l':
+	if (edit)
+		res = do_edit(user_idnr, script_name);
+	if (activate)
+		/* Don't activate the script if it wasn't inserted/edited */
+		if (!(insert && res) && !(edit && res))
+			res = do_activate(user_idnr, script_name);
+	if (deactivate)
+		res = do_deactivate(user_idnr, script_name);
+	if (list)
 		res = do_list(user_idnr);
-		break;
-	case 'h':
-	default:
+	if (cat)
+		res = do_cat(user_idnr, script_name, stdout);
+	if (help)
 		res = do_showhelp();
-		break;
-	}
 
       mainend:
 	g_free(user_name);
@@ -249,6 +273,11 @@ int do_activate(u64_t user_idnr, char *name)
 {
 	int res = 0;
 
+	if (!name) {
+		qerrorf("Must give the name of a script to activate.\n");
+		return -1;
+	}
+
 	res = db_activate_sievescript(user_idnr, name);
 	if (res == -3) {
 		qerrorf("Script [%s] does not exist.\n", name);
@@ -269,6 +298,21 @@ int do_activate(u64_t user_idnr, char *name)
 int do_deactivate(u64_t user_idnr, char *name)
 {
 	int res = 0;
+	char *scriptname = NULL;
+
+	if (!name) {
+		if (db_get_sievescript_active(user_idnr, &scriptname)) {
+			qerrorf("Database error when fetching active script.\n");
+			return -1;
+		}
+		
+		if (scriptname == NULL) {
+			qerrorf("No active script found.\n");
+			return -1;
+		}
+
+		name = scriptname;
+	}
 
 	res = db_deactivate_sievescript(user_idnr, name);
 	if (res == -3) {
@@ -282,10 +326,121 @@ int do_deactivate(u64_t user_idnr, char *name)
 		" No scripts are currently active.\n",
 		name);
 
+	g_free(scriptname);
+
 	return 0;
 }
 
-int do_cat(u64_t user_idnr, char *name)
+int do_edit(u64_t user_idnr, char *name)
+{
+	int ret = 0;
+	int editor_val = 0;
+	int old_yes_to_all = yes_to_all;
+	char *tmp = NULL, *editor = NULL, *editor_cmd = NULL;
+	FILE *ftmp = NULL;
+	char *scriptname = NULL;
+	struct stat stat_before, stat_after;
+
+	if (!name) {
+		if (db_get_sievescript_active(user_idnr, &scriptname)) {
+			qerrorf("Database error when fetching active script!\n");
+			ret = 1;
+			goto cleanup;
+		}
+		
+		if (scriptname == NULL) {
+			qerrorf("No active script found!\n");
+			ret = 1;
+			goto cleanup;
+		}
+
+		name = scriptname;
+	}
+
+	/* Check for the EDITOR environment variable. */
+	editor = getenv("EDITOR");
+
+	if (!editor) {
+		qerrorf("No EDITOR environment variable.\n");
+		ret = 1;
+		goto cleanup;
+	}
+
+	/* Open a temp file. */
+	if (!(tmp = tempnam(NULL, "dbmail"))) {
+		qerrorf("Could not make temporary file name: %s\n", strerror(errno));
+		ret = 1;
+		goto cleanup;
+	}
+
+	if (!(ftmp = fopen(tmp, "w+"))) {
+		qerrorf("Could not open temporary file [%s]: %s\n", tmp, strerror(errno));
+		ret = 1;
+		goto cleanup;
+	}
+
+	/* do_cat the script. */
+	if (do_cat(user_idnr, name, ftmp)) {
+		qerrorf("Could not dump script [%s] to temporary file.\n", name);
+		ret = 1;
+		goto cleanup;
+	}
+
+	/* Make sure that the file is written before we call the editor. */
+	fflush(ftmp);
+	fstat(fileno(ftmp), &stat_before);
+
+	/* Call the editor. */
+	editor_cmd = g_strdup_printf("%s %s", editor, tmp);
+	if ((editor_val = system(editor_cmd))) {
+		qerrorf("Execution of EDITOR [%s] returned non-zero [%d].\n", editor, editor_val);
+		ret = 1;
+		goto cleanup;
+	}
+
+	fstat(fileno(ftmp), &stat_after);
+
+	/* If the file does not appear to have changed, cancel insertion. */
+	if ((stat_before.st_mtime == stat_after.st_mtime)
+	 && (stat_before.st_size == stat_after.st_size)) {
+		qprintf("File not modified, canceling.\n");
+		ret = 0;
+		goto cleanup;
+	}
+
+	/* do_insert the script (set yes_to_all as we will overwrite). */
+	yes_to_all = 1;
+	if (do_insert(user_idnr, name, tmp)) {
+		char dbmail_invalid_file[] = "dbmail-invalid.xxx.sieve";
+		struct stat stat_inv;
+		int i;
+		/* Save the script locally. */
+		for (i = 0; i < 20; i++) {
+			sprintf(dbmail_invalid_file, "dbmail-invalid.%d.sieve", i);
+			if (stat(dbmail_invalid_file, &stat_inv)) {
+				/* Try to rename the tmp file to this unused name. */
+				if (rename(tmp, dbmail_invalid_file) == 0)
+					break;
+				else
+					qerrorf("Could not save script to [%s]: %s\n", dbmail_invalid_file, strerror(errno));
+			}
+		}
+		qerrorf("Saved script to [%s]\n", dbmail_invalid_file);
+	}
+
+	/* Ok, all done. */
+cleanup:
+	g_free(scriptname);
+	g_free(editor_cmd);
+	g_free(tmp);
+	if (ftmp) fclose(ftmp);
+	if (tmp) unlink(tmp);
+	yes_to_all = old_yes_to_all;
+
+	return ret;
+}
+
+int do_cat(u64_t user_idnr, char *name, FILE *out)
 {
 	int res = 0;
 	char *buf = NULL;
@@ -318,7 +473,7 @@ int do_cat(u64_t user_idnr, char *name)
 		return -1;
 	}
 
-	printf("%s", buf);
+	fputs(buf, out);
 
 	g_free(buf);
 	if (!name)
@@ -473,26 +628,39 @@ int do_list(u64_t user_idnr)
 
 int do_showhelp(void)
 {
-	printf("*** dbmail-sievecmd ***\n");
+	printf(
+	"*** dbmail-sievecmd ***\n"
+//	Try to stay under the standard 80 column width
+//	0........10........20........30........40........50........60........70........80
+	"Use this program to manage your users' Sieve scripts.\n"
+	"See the man page for more info. Summary:\n\n"
+	"     -u username            Username of script user \n"
+	"     -l                     List scripts belonging to user \n"
+	"     -a scriptname          Activate the named script \n"
+	"                            (only one script can be active; \n"
+	"                             deactivates any others) \n"
+	"     -d [scriptname]        Deactivate the named script \n"
+	"     -c [scriptname]        Print the contents of the named script\n"
+	"     -e [scriptname]        Edit the contents of the named script\n"
+	"                            (if no script is given, the active \n"
+	"                             script is printed) \n"
+	"     -i scriptname file     Insert the named script from file \n"
+	"                            (a single dash, -, indicates input \n"
+	"                             from STDIN) \n"
+	"     -r scriptname          Remove the named script \n"
+	"                            (if script was active, no script is \n"
+	"                             active after deletion) \n"
 
-	printf("Use this program to manage your users' Sieve scripts.\n");
-	printf("See the man page for more info. Summary:\n\n");
-	printf("     -u username            Username of script user \n");
-	printf("     -l                     List scripts belonging to user \n");
-	printf("     -a scriptname          Activate the named script \n");
-	printf("                            (only one script can be active; \n"
-	       "                             deactivates any others) \n");
-	printf("     -d scriptname          Deactivate the named script \n");
-	printf("                            (no scripts will be active after this) \n");
-	printf("     -i scriptname file     Insert the named script from file \n");
-	printf("                            (a single dash, -, indicates input \n"
-	       "                             from STDIN) \n");
-	printf("     -c [scriptname]        Print the contents of the named script\n");
-	printf("                            (if no script is given, the active \n"
-	       "                             script is printed) \n");
-	printf("     -r scriptname          Remove the named script \n");
-	printf("                            (if script was active, no script is \n"
-	       "                             active after deletion) \n");
+        "\nCommon options for all DBMail utilities:\n"
+	"     -f file   specify an alternative config file\n"
+	"     -q        quietly skip interactive prompts\n"
+	"               use twice to suppress error messages\n"
+	"     -n        show the intended action but do not perform it, no to all\n"
+	"     -y        perform all proposed actions, as though yes to all\n"
+	"     -v        verbose details\n"
+	"     -V        show the version\n"
+	"     -h        show this help message\n"
+	);
 
 	return 0;
 }
