@@ -210,14 +210,13 @@ class testImapServer(unittest.TestCase):
         p.select('testexpungebox')
         
         self.o.store('5:*', '+FLAGS', '\Deleted')
-        self.o.debug = 4
-        msnlist = self.o.expunge()[1];
-        self.assertEquals(msnlist,['11', '10', '9', '8', '7', '6', '5'])
+        msnlist = self.o.expunge()[1]
+        self.assertEquals(msnlist,['11','10','9','8','7','6','5'])
         self.assertEquals(self.o.fetch('4','(BODYSTRUCTURE FLAGS)')[0],'OK')
         
-        p.debug = 4
+        self.assertEquals(self.o.fetch('4','(BODYSTRUCTURE FLAGS)')[0],'OK')
+        self.assertEquals(p.fetch('11','(Flags)')[0],'OK')
         print p.noop()
-
         self.o.debug = 0
         p.debug = 0
 
@@ -281,6 +280,12 @@ class testImapServer(unittest.TestCase):
         self.o.select('INBOX')
         self.o.fetch("1:*","(UID FULL)")
 
+        self.assertEquals(self.o.fetch("1:*","(Flags)")[0],'OK')
+
+        # bogus ranges
+        self.assertRaises(self.o.error,self.o.fetch,"-10:10","(Flags)")
+        self.assertRaises(self.o.error,self.o.fetch,"10:-10","(Flags)")
+
     def testGetacl(self):
         """ 
         `getacl(mailbox)'
